/** 
	Copyright 2009 Bill Horne

	This file is part of the Poincare program for photographic
	tiling of the Poincare disk.

    Poincare is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Poincare is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Poincare.  If not, see <http://www.gnu.org/licenses/>.
**/

#include <cmath>
#include "AffineTransform.h"

AffineTransform::AffineTransform()
{
	_a00 = 1.0;
	_a01 = 0.0;
	_a02 = 0.0;
	_a10 = 0.0;
	_a11 = 1.0;
	_a12 = 0.0;
}

Point
AffineTransform::transform(Point p)
{
	Point q;
	q.x = _a00 * p.x + _a01 * p.y + _a02;
	q.y = _a10 * p.x + _a11 * p.y + _a12;
	return q;
}

void
AffineTransform::translate(double x, double y)
{
	_a02 -= x;
	_a12 -= y;
}

void
AffineTransform::rotate(double theta)
{
	double a00 = _a00 * cos(theta) - _a10 * sin(theta);
	double a01 = _a01 * cos(theta) - _a11 * sin(theta);
	double a02 = _a02 * cos(theta) - _a12 * sin(theta);

	double a10 = _a00 * sin(theta) + _a10 * cos(theta);
	double a11 = _a01 * sin(theta) + _a11 * cos(theta);
	double a12 = _a02 * sin(theta) + _a12 * cos(theta);

	_a00 = a00;
	_a01 = a01;
	_a02 = a02;
	_a10 = a10;
	_a11 = a11;
	_a12 = a12;
}

void
AffineTransform::scale(double s)
{
	_a00 *= s;
	_a01 *= s;
	_a02 *= s;
	_a10 *= s;
	_a11 *= s;
	_a12 *= s;
}
