/** 
	Copyright 2009 Bill Horne

	This file is part of the Poincare program for photographic
	tiling of the Poincare disk.

    Poincare is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Poincare is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Poincare.  If not, see <http://www.gnu.org/licenses/>.
**/

#include <cmath>
#include <cstdio>
#include "Circle.h"

Circle::Circle()
{
	_r = 1.0;
	_p.x = 0.0;
	_p.y = 0.0;
}

Circle::Circle(Point p, double r)
{
	_r = r;
	_p = p;
}


bool
Circle::insideOrOn(Point p)
{
	double x = _p.x - p.x;
	double y = _p.y - p.y;

	return (x*x + y*y <= _r*_r) ? true : false;
}

Point
Circle::invert(Point p)
{
	Point q;
	double dx = p.x - _p.x;
	double dy = p.y - _p.y;
	double rSquared = dx*dx + dy*dy;
	double r2 = _r * _r / rSquared;
	double x2 = r2 * dx;
	double y2 = r2 * dy;
	q.x = _p.x + x2;
	q.y = _p.y + y2;
	return q;
}

Circle
createEdgeCircle(int p, int q)
{
	double cosq2 = cos(PI/((double)q))*cos(PI/((double)q));
	double sinp2 = sin(PI/((double)p))*sin(PI/((double)p));

	Point c;
	c.x = sqrt(cosq2/(cosq2 - sinp2));
	c.y = 0.0;

	double r = sqrt(sinp2/(cosq2 - sinp2));

	return Circle(c, r);
}