/** 
	Copyright 2009 Bill Horne

	This file is part of the Poincare program for photographic
	tiling of the Poincare disk.

    Poincare is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Poincare is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Poincare.  If not, see <http://www.gnu.org/licenses/>.
**/

#include <cmath>
#include <cstdio>
#include "Circle.h"
#include "FundamentalRegion.h"

FundamentalRegion::FundamentalRegion(int p, int q)
{
	_p = p;
	_edge = createEdgeCircle(p, q);
	_m = tan(PI/(double)p);
	_c = cos(2.0 * PI/(double)p);
	_s = sin(2.0 * PI/(double)p);
}

bool
FundamentalRegion::inside(Point p)
{
	if (p.y > _m * p.x ) return false;
	if (p.y < 0.0) return false;
	if (_edge.insideOrOn(p)) return false;
	return true;
}

// This is the reverse pixel lookup routing.  Parity is useful
// for doing things like creating tessellation diagrams where
// alternating triangles are different colors.
bool
FundamentalRegion::reversePixelLookup(Point a, int maxIterations, Point *out, bool *parity)
{
	int d = maxIterations;

	Point b = a;
	while (d >= 0) {


#if 0
		// Note:  this is more efficient than the below code, but
		// it can get into an infinite loop, which appears to be
		// due to some kind of numerical precision problem.

		// rotate point to within +/- PI/p
		// is it within the cone defined by +/- _m?
		Point c = b;
		while ((c.y > _m * c.x) || (c.y < -_m * c.x)) {
			// apply a linear rotation transformation
			double tmp = _c * c.x - _s * c.y;
			c.y = _s * c.x + _c * c.y;
			c.x = tmp;
		}
#else
		// rotate point to within +/- PI/p
		// TODO:  Make this more efficient!
		double theta = atan2(b.y, b.x);
		double r = sqrt(b.x*b.x + b.y*b.y);
		while (theta < 0.0) theta += 2.0 * PI;
		while (theta > PI/(double)_p) {
			theta -= 2.0*PI/(double)_p;
		}
		Point c;
		c.x = r * cos(theta);
		c.y = r * sin(theta);
#endif

		if (c.y < 0.0) {
			c.y = -c.y;
			*parity = (*parity) ? false : true;
		}

		if (inside(c)) {
			out->x = c.x;
			out->y = c.y;
			return false;
		}

		b = _edge.invert(c);
		*parity = (*parity) ? false : true;
		if (inside(b)) {
			out->x = b.x;
			out->y = b.y;
			return false;
		}
		d--;
	}
	
	return true;
}
