/** 
	Copyright 2009 Bill Horne

	This file is part of the Poincare program for photographic
	tiling of the Poincare disk.

    Poincare is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Poincare is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Poincare.  If not, see <http://www.gnu.org/licenses/>.
**/

#include <cmath>
#include <cstring>
#include "ImageProxy.h"

PixelProxy::PixelProxy(RGBQUAD pixel)
{
	_pixel = pixel;
}

PixelProxy::PixelProxy(unsigned r, unsigned g, unsigned b)
{
	_pixel.rgbRed = r;
	_pixel.rgbGreen = g;
	_pixel.rgbBlue = b;
}

// TODO:  Give the user the option of specifying a color as #xxyyzz
PixelProxy::PixelProxy(char *color)
{
	if (!strcmp(color, "white")) {
		_pixel.rgbRed = 255;
		_pixel.rgbGreen = 255;
		_pixel.rgbBlue = 255;
	} else if (!strcmp(color, "black")) {
		_pixel.rgbRed = 0;
		_pixel.rgbGreen = 0;
		_pixel.rgbBlue = 0;
	} else if (!strcmp(color, "red")) {
		_pixel.rgbRed = 255;
		_pixel.rgbGreen = 0;
		_pixel.rgbBlue = 0;
	} else if (!strcmp(color, "green")) {
		_pixel.rgbRed = 0;
		_pixel.rgbGreen = 255;
		_pixel.rgbBlue = 0;
	} else if (!strcmp(color, "blue")) {
		_pixel.rgbRed = 0;
		_pixel.rgbGreen = 0;
		_pixel.rgbBlue = 255;
	} else if (!strcmp(color, "yellow")) {
		_pixel.rgbRed = 255;
		_pixel.rgbGreen = 255;
		_pixel.rgbBlue = 0;
	} else if (!strcmp(color, "magenta")) {
		_pixel.rgbRed = 255;
		_pixel.rgbGreen = 0;
		_pixel.rgbBlue = 255;
	} else if (!strcmp(color, "cyan")) {
		_pixel.rgbRed = 0;
		_pixel.rgbGreen = 255;
		_pixel.rgbBlue = 255;
	} else {
		throw "unknown color";
	}
}

void PixelProxy::negate() {
	_pixel.rgbRed = 255 - _pixel.rgbRed;
	_pixel.rgbGreen = 255 - _pixel.rgbGreen;
	_pixel.rgbBlue = 255 - _pixel.rgbBlue;
}


ImageProxy::ImageProxy(char *filename)
{
	_bitmap = FreeImage_Load(FIF_JPEG, filename, JPEG_ACCURATE);
	_width = FreeImage_GetWidth(_bitmap);
	_height = FreeImage_GetHeight(_bitmap);
}

ImageProxy::ImageProxy(unsigned width, unsigned height)
{
	_bitmap = FreeImage_Allocate(width, height, 24);
	_width = width;
	_height = height;
}

int ImageProxy::getWidth() const { return _width; }
int ImageProxy::getHeight() const { return _height; }

PixelProxy*
ImageProxy::getPixel(unsigned x, unsigned y)
{
	RGBQUAD pixel;
	FreeImage_GetPixelColor(_bitmap, x, y, &pixel);
	return new PixelProxy(pixel);
}

#if 0
// This just rounds to the nearest pixel coordinate
PixelProxy*
ImageProxy::getSubPixel(double x, double y)
{
    // Round to nearest pixel
    unsigned x1 = (unsigned)floor(x+0.5);
    unsigned y1 = (unsigned)floor(y+0.5);

    // pixel out of bounds
	if ((x1 < 0) || (x1 >= _width) || (y1 < 0) || (y1 >= _height)) {
        return 0;
    }

	RGBQUAD pixel1;
	FreeImage_GetPixelColor(_bitmap, x1, y1, &pixel1);
	PixelProxy *pp = new PixelProxy((unsigned)pixel1.rgbRed, (unsigned)pixel1.rgbGreen, (unsigned)pixel1.rgbBlue);
	return pp;
}
#else
// This uses bilinear interpolation
PixelProxy*
ImageProxy::getSubPixel(double x, double y)
{
	double r1, r2;

    // Bilinear interpolate around subpixel q in image1 to get value
    unsigned x1 = (unsigned)floor(x);
    unsigned x2 = x1 + 1;
    
    unsigned y1 = (unsigned)floor(y);
    unsigned y2 = y1 + 1;

    // pixel out of bounds
	if ((x1 < 0) || (x2 >= _width) || (y1 < 0) || (y2 >= _height)) {
        return 0;
    }

	// Do bilinear interpolation
	RGBQUAD pixel1;
	RGBQUAD pixel2;
	RGBQUAD pixel3;
	RGBQUAD pixel4;

	FreeImage_GetPixelColor(_bitmap, x1, y1, &pixel1);
	FreeImage_GetPixelColor(_bitmap, x1, y2, &pixel2);
	FreeImage_GetPixelColor(_bitmap, x2, y1, &pixel3);
	FreeImage_GetPixelColor(_bitmap, x2, y2, &pixel4);

	// red
    r1 = (double)pixel1.rgbRed * ((double)x2 - x) + (double)pixel3.rgbRed * (x - (double)x1);
    r2 = (double)pixel2.rgbRed * ((double)x2 - x) + (double)pixel4.rgbRed * (x - (double)x1);
    double dred = r1*((double)y2 - y) + r2*(y - (double)y1);
	if (dred > 255.0) dred = 255.0;
    if (dred < 0.0) dred = 0.0;

	// green
    r1 = (double)pixel1.rgbGreen * ((double)x2 - x) + (double)pixel3.rgbGreen * (x - (double)x1);
    r2 = (double)pixel2.rgbGreen * ((double)x2 - x) + (double)pixel4.rgbGreen * (x - (double)x1);
    double dgreen = r1*((double)y2 - y) + r2*(y - (double)y1);
    if (dgreen > 255.0) dgreen = 255.0;
    if (dgreen < 0.0) dgreen = 0.0;

	// blue
    r1 = (double)pixel1.rgbBlue * ((double)x2 - x) + (double)pixel3.rgbBlue * (x - (double)x1);
    r2 = (double)pixel2.rgbBlue * ((double)x2 - x) + (double)pixel4.rgbBlue * (x - (double)x1);
    double dblue = r1*((double)y2 - y) + r2*(y - (double)y1);
    if (dblue > 255.0) dblue = 255.0;
    if (dblue < 0.0) dblue = 0.0;

	PixelProxy *pp = new PixelProxy((unsigned)dred, (unsigned)dgreen, (unsigned)dblue);
	return pp;
}
#endif

void
ImageProxy::setPixel(unsigned x, unsigned y, PixelProxy* pixel)
{
	RGBQUAD value = pixel->rgbquad();
	FreeImage_SetPixelColor(_bitmap, x, y, &value);
}

void
ImageProxy::write(char *filename)
{
	FreeImage_Save(FIF_JPEG, _bitmap, filename, JPEG_QUALITYSUPERB);
}