/** 
	Copyright 2009 Bill Horne

	This file is part of the Poincare program for photographic
	tiling of the Poincare disk.

    Poincare is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Poincare is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Poincare.  If not, see <http://www.gnu.org/licenses/>.
**/

#ifndef _ImageProxy_h_
#define _ImageProxy_h_

#include "FreeImage.h"

class PixelProxy
{
public:
	PixelProxy(RGBQUAD pixel);
	PixelProxy(unsigned r, unsigned g, unsigned b);
	PixelProxy(char *color);

	RGBQUAD rgbquad() const { return _pixel; }
	
	int red() const { return _pixel.rgbRed; }
	int green() const { return _pixel.rgbGreen; }
	int blue() const { return _pixel.rgbBlue; }

	void setRed(int r) { _pixel.rgbRed = r; }
	void setGreen(int g) { _pixel.rgbGreen = g; }
	void setBlue(int b) { _pixel.rgbBlue = b; }

	void negate();

private:
	RGBQUAD _pixel;
};

class ImageProxy
{
public:
	ImageProxy(char *filename);
	ImageProxy(unsigned width, unsigned height);

	int getWidth() const;
	int getHeight() const;

	PixelProxy* getPixel(unsigned x, unsigned y);
	PixelProxy* getSubPixel(double x, double y);
	void setPixel(unsigned x, unsigned y, PixelProxy* pixel);

	void write(char *filename);
private:

	FIBITMAP *_bitmap;
	unsigned _width;
	unsigned _height;
};

#endif // _ImageProxy_h_