/** 
	Copyright 2009 Bill Horne

	This file is part of the Poincare program for photographic
	tiling of the Poincare disk.

    Poincare is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Poincare is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Poincare.  If not, see <http://www.gnu.org/licenses/>.
**/

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include "Parameters.h"
#include "Properties.h"

Parameters::Parameters(char *filename)
{
	char *value;
	Properties *prop = new Properties(filename);

	value = prop->getProperty("p");
	if (value == 0) { throw "p not specified in properties file"; }
	p = atoi(value);

	value = prop->getProperty("q");
	if (value == 0) { throw "q not specified in properties file"; }
	q = atoi(value);

	value = prop->getProperty("jpegQuality");
	if (value == 0) { 
		jpegQuality = 100;
	} else {
		jpegQuality = atoi(value);
	}

	infile = prop->getProperty("infile");
	if (infile == 0) { throw "infile not specified in properties file"; }

	outfile = prop->getProperty("outfile");
	if (outfile == 0) { throw "outfile not specified in properties file"; }

	action = prop->getProperty("action");
	if (action == 0) { throw "action not specified in properties file"; }

	value = prop->getProperty("px");
	if (value == 0) { throw "px not specified in properties file"; }
	px = atof(value);

	value = prop->getProperty("py");
	if (value == 0) { throw "py not specified in properties file"; }
	py = atof(value);

	value = prop->getProperty("rx");
	if (value == 0) { throw "rx not specified in properties file"; }
	rx = atof(value);

	value = prop->getProperty("ry");
	if (value == 0) { throw "ry not specified in properties file"; }
	ry = atof(value);

	// FundamentalRegion Parameters
	maskColor = prop->getProperty("maskColor");

	// Render Parameters
	value = prop->getProperty("renderedSize");
	if (value == 0) { throw "renderedSize not specified in properties file"; }
	renderedSize = atoi(value);

	value = prop->getProperty("borderSize");
	if (value == 0) { throw "borderSize not specified in properties file"; }
	borderSize = atoi(value);

	value = prop->getProperty("supersamplingRate");
	if (value == 0) { throw "supersamplingRate not specified in properties file"; }
	supersamplingRate = atoi(value);

	value = prop->getProperty("perturbationFactor");
	if (value == 0) { throw "perturbationFactor not specified in properties file"; }
	perturbationFactor = atof(value);

	outOfBoundsColor = prop->getProperty("outOfBoundsColor");

	unreachableColor = prop->getProperty("unreachableColor");

	nonSourceColor = prop->getProperty("nonSourceColor");

	value = prop->getProperty("maxIterations");
	if (value == 0) { throw "maxIterations not specified in properties file"; }
	maxIterations = atoi(value);

	value = prop->getProperty("negate");
	if (strcmp(value, "true") == 0) negate = true;
	else if (strcmp(value, "false") == 0) negate = false;
	else throw "bad value for negate in properties file";

	// TODO:  Need to verify all required parameters were set, or set default values	
}

void
Parameters::print()
{
	printf("p = %d\n", p);
	printf("q = %d\n", q);
	printf("jpegQuality = %d\n", jpegQuality);
	printf("infile = %s\n", infile);
	printf("outfile = %s\n", outfile);
	printf("action = %s\n", action);
	printf("upper = %s\n", upper ? "true" : "false");
	printf("px = %f\n", px);
	printf("py = %f\n", py);
	printf("rx = %f\n", rx);
	printf("ry = %f\n", ry);
	
	// FundamentalRegion Parameters
	printf("maskColor = %s\n", maskColor);

	// Render Parameters
	printf("renderedSize = %d\n", renderedSize);
	printf("borderSize = %d\n", borderSize);
	printf("perturbationFactor = %f\n", perturbationFactor);
	printf("outOfBoundsColor = %s\n", outOfBoundsColor);
	printf("unreachableColor = %s\n", unreachableColor);
	printf("nonSourceColor = %s\n", nonSourceColor);
	printf("maxIterations = %d\n", maxIterations);
	printf("negate = %s\n", negate ? "true" : "false");
}